#! /usr/bin/env node
'use strict';

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _checkUpdate = require('check-update');

var _checkUpdate2 = _interopRequireDefault(_checkUpdate);

var _cli = require('./cli');

var _cli2 = _interopRequireDefault(_cli);

var _log = require('./utils/log');

var _log2 = _interopRequireDefault(_log);

var _config = require('./config');

var _config2 = _interopRequireDefault(_config);

var _analytics = require('./utils/analytics');

var _analytics2 = _interopRequireDefault(_analytics);

var _uriOnlineJudge = require('./uri/uri-online-judge');

var _uriOnlineJudge2 = _interopRequireDefault(_uriOnlineJudge);

var _uriOnlineJudgeUrls = require('./uri/uri-online-judge-urls');

var _uriOnlineJudgeUrls2 = _interopRequireDefault(_uriOnlineJudgeUrls);

var _package = require('../package.json');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

require('babel-polyfill');

main();

function main() {
  // always check for updates
  checkForUpdate().then(function () {
    return command();
  });
}

function command() {
  var commands = {
    reset: reset,
    submit: submit,
    init: init
  };

  if (commands[_cli2.default.command] !== undefined) {
    commands[_cli2.default.command]()
    // flush every analytics
    .then(function () {
      return _analytics2.default.flush();
    }).catch(function (error) {
      _log2.default.error(error);
    })
    // make sure the process is finished.
    .then(process.exit);
  } else {
    _log2.default.error('Command \'' + _cli2.default.command + '\' not found.');
  }
}

/**
 * Reset user informations
 */
function reset() {
  return _config2.default.load(true).then(function (config) {
    _log2.default.success('Email: ' + config.email, 'Template: ' + config.template);
  }).catch(function (error) {
    _log2.default.error(error);
  });
}

/**
 * Load user and submit a problem to URI Online Judge Website
 */
function submit() {
  var problemFile = _fs2.default.readFileSync(_cli2.default.filepath, 'utf-8');
  var ext = _path2.default.extname(_cli2.default.filepath);
  var number = _cli2.default.number ? _cli2.default.number : _path2.default.basename(_cli2.default.filepath, ext);

  return _config2.default.load()
  // submit problem to the uri online judge website
  .then(function (config) {
    return _uriOnlineJudge2.default.submit({
      email: config.email,
      password: config.password,
      problem: parseInt(number, 10),
      file: problemFile,
      language: _cli2.default.language
    });
  })
  // send success analytics
  .then(function (answer) {
    return _analytics2.default.submit({
      problem: number,
      result: answer
    });
  })
  // send error analytics
  .catch(function (error) {
    return _analytics2.default.error({
      command: 'submit',
      problem: number,
      error: error.stack ? error.stack : error
    }).then(function () {
      throw new Error(error);
    });
  });
}

/**
 * Init a question from the URI Online Judge Website
 */
function init() {
  var force = _cli2.default.force;
  var problemNumber = _cli2.default.number;
  var injectValue = 'urionlinejudge::description';

  return _config2.default.load().then(function (config) {
    var template = _cli2.default.template ? _cli2.default.template : config.template;

    var extname = _path2.default.extname(template);
    var outputFilepath = _path2.default.join(_cli2.default.output, '' + problemNumber + extname);
    var templateFile = _fs2.default.readFileSync(template, 'utf-8');

    return checkTemplate({
      force: force,
      injectValue: injectValue,
      templateFile: templateFile,
      problemNumber: problemNumber,
      outputFilepath: outputFilepath
    }).then(function () {
      var desc = _uriOnlineJudgeUrls2.default.problemView + problemNumber;
      var outputFile = templateFile.replace(injectValue, desc);

      _fs2.default.writeFileSync(outputFilepath, outputFile);

      _log2.default.success('Problem init: ' + outputFilepath);
    });
  });
}

/**
 * Check if template already exists and reject if force is false.
 * Check if template contains INJECT_VALUE, reject if don't.
 */
function checkTemplate(_ref) {
  var injectValue = _ref.injectValue;
  var templateFile = _ref.templateFile;
  var templateFilepath = _ref.templateFilepath;
  var outputFilepath = _ref.outputFilepath;
  var problemNumber = _ref.problemNumber;
  var force = _ref.force;

  return new Promise(function (resolve, reject) {
    var exists = true;

    try {
      _fs2.default.accessSync(outputFilepath);
    } catch (e) {
      exists = false;
    }

    // Verify if template contains INJECT_VALUE
    if (templateFile.indexOf(injectValue) === -1) {
      reject(['Can\'t find the inject string.', 'Make sure that your template contains \'' + injectValue + '\'.']);
      // reject if force is false and file already exists
    } else if (!force && exists) {
        reject(['This file ' + outputFilepath + ' already exists. Use -f or --force if you want to overwrite:', 'urionlinejudge fetch -f -n ' + problemNumber + ' -t ' + templateFilepath + ' -o ' + outputFilepath]);
      } else {
        resolve();
      }
  });
}

/**
 * Check for CLI updates (on npm) and print some warning on the command line.
 */
function checkForUpdate() {
  return new Promise(function (resolve) {
    if (process.env.DEBUG) {
      return resolve();
    }

    (0, _checkUpdate2.default)({
      packageName: _package.name,
      packageVersion: _package.version,
      isCLI: true
    }, function (err, latestVersion, defaultMessage) {
      if (!err && _package.version !== latestVersion) {
        console.log(defaultMessage + '\n');
      }
      resolve();
    });
  });
}